﻿using System.Collections.Generic;
using Data;
using UnityEngine;
using Utilities;

namespace Interactables
{
    public abstract class BaseInteractable : MonoBehaviour
    {
        [SerializeField]
        private bool isLocked;

        [SerializeField] private ColorShifterType colorShifterType;

        [SerializeField] private LightColor[] shiftedLightColors;

        [SerializeField] private ColorFilterType colorFilterType;

        [SerializeField] private LightColor[] filterColors;

        [SerializeField]
        private MeshRenderer lockedMeshRenderer;

        [SerializeField]
        private MeshRenderer [] filteredColorMeshRenderers;

        [SerializeField]
        private Material convertedColorMaterial;

        [SerializeField]
        private MeshRenderer[] convertedColorMeshRenderers;

        [SerializeField]
        private Material filterColorMaterial;

        private BaseColorFilter colorFilter;

        private BaseColorShifter colorShifter;

        protected BaseColorFilter ColorFilter
        {
            get { return colorFilter; }
        }

        protected BaseColorShifter ColorShifter
        {
            get { return colorShifter; }
        }

        public bool IsLocked
        {
            get { return isLocked; }
        }

        private void Start()
        {
            if (this.lockedMeshRenderer != null)
            {
                this.lockedMeshRenderer.enabled = this.IsLocked;
            }

            for (int i = 0; i < this.filteredColorMeshRenderers.Length; i++)
            {
                MeshRenderer filterColorMeshRenderer = filteredColorMeshRenderers[i];
                Material filterColorMaterialClone = Instantiate<Material>(this.filterColorMaterial);
                filterColorMeshRenderer.material = filterColorMaterialClone;
                filterColorMeshRenderer.enabled = false;

                if (i < this.filterColors.Length && this.colorFilterType != ColorFilterType.None)
                {
                    filteredColorMeshRenderers[i].material.color = GameUtilities.LightColorDatabase[this.filterColors[i]];
                    filteredColorMeshRenderers[i].enabled = true;
                }
            }

            for (int i = 0 ; i < this.convertedColorMeshRenderers.Length; i++)
            {
                MeshRenderer convertedColorMeshRenderer = convertedColorMeshRenderers[i];
                Material convertedColorMaterialClone = Instantiate<Material>(this.convertedColorMaterial);
                convertedColorMeshRenderer.material = convertedColorMaterialClone;
                convertedColorMeshRenderer.enabled = false;

                if (i < this.shiftedLightColors.Length && this.colorShifterType != ColorShifterType.None)
                {
                    convertedColorMeshRenderers[i].material.color = GameUtilities.LightColorDatabase[this.shiftedLightColors[i]];
                    convertedColorMeshRenderers[i].enabled = true;
                }
            }

            this.InitializeColorShiftersAndColorFilters();
            this.Initialize();
        }

        protected virtual void Initialize()
        {
            
        }

        private void InitializeColorShiftersAndColorFilters()
        {
            this.colorShifter = ColorShifterFactory.CreateColorShifter(colorShifterType, new List<LightColor>(this.shiftedLightColors));
            this.colorFilter = ColorFilterFactory.CreateColorFilter(colorFilterType, new List<LightColor>(this.filterColors));

            //Debug.LogWarning("InitializeColorShiftersAndColorFilters: " + this.name + " - " + (this.colorShifter == null) + " - " + (this.colorFilter == null));
        }

        public abstract List<LightRay> RetrieveDirectedRayList(LightRay incoming,
            Vector3 rayHitPosition);

        public bool AcceptColor(LightColor lightColor)
        {
            return this.colorFilter.Accept(lightColor);
        }

        public List<LightColor> ConvertColor(LightColor lightColor)
        {
            return this.colorShifter.Shift(lightColor);
        }

        public abstract bool Raycast(Ray incomingRay, out RaycastHit hitInfo);
    }
}
