﻿using System;
using System.Collections;
using System.Collections.Generic;
using Data;
using UnityEngine;

namespace Interactables
{
    public class Gate : BaseInteractable
    {
        [SerializeField]
        private GameObject gate1;

        [SerializeField]
        private GameObject gate2;

        [SerializeField]
        private GameObject gate1Halo;

        [SerializeField]
        private GameObject gate2Halo;

        [SerializeField]
        private Material haloMaterial;

        [SerializeField]
        private Color haloColor;

        private GameObject inputGate, outputGate;

        protected override void Initialize()
        {
            MeshRenderer meshRenderer = this.gate1Halo.GetComponent<MeshRenderer>();
            Material haloMaterialClone = Instantiate<Material>(this.haloMaterial);
            haloMaterialClone.color = this.haloColor;
            meshRenderer.material = haloMaterialClone;

            meshRenderer = this.gate2Halo.GetComponent<MeshRenderer>();
            haloMaterialClone = Instantiate<Material>(this.haloMaterial);
            haloMaterialClone.color = this.haloColor;
            meshRenderer.material = haloMaterialClone;
        }

        public override bool Raycast(Ray incomingRay, out RaycastHit hitInfo)
        {
            Collider[] childColliders = this.GetComponentsInChildren<Collider>();
            hitInfo = new RaycastHit();
            float closestColliderDistanceToRayOrigin = Mathf.Infinity;
            Collider closestColliderToRayOrigin = null;
            RaycastHit closestHitInfo = new RaycastHit();
            foreach (Collider childCollider in childColliders)
            {
                if (childCollider.Raycast(incomingRay, out hitInfo, Mathf.Infinity))
                {
                    float currentColliderToOriginDistance = Vector3.Distance(incomingRay.origin, childCollider.transform.position);
                    if (currentColliderToOriginDistance < closestColliderDistanceToRayOrigin)
                    {
                        closestColliderDistanceToRayOrigin = currentColliderToOriginDistance;
                        closestColliderToRayOrigin = childCollider;
                        closestHitInfo = hitInfo;
                    }
                }
            }

            if (closestColliderToRayOrigin != null)
            {
                inputGate = (closestColliderToRayOrigin == gate1.GetComponent<Collider>()) ? gate1 : gate2;
                outputGate = (closestColliderToRayOrigin == gate1.GetComponent<Collider>()) ? gate2 : gate1;
                hitInfo = closestHitInfo;

                //Debug.Log(inputGate.name + " " + outputGate.name);
                return true;
            }

            return false;
        }

        public override List<LightRay> RetrieveDirectedRayList(LightRay incoming,
            Vector3 rayHitPosition)
        {
            List<LightRay> reflectedRayList = new List<LightRay>();

            Vector3 normal = inputGate.transform.right;
            float dotNormal = Vector3.Dot(incoming.ray.direction, normal);

            bool isColorAccepted = this.AcceptColor(incoming.lightColor);
            if (dotNormal >= 0 || !isColorAccepted)
            {
                return reflectedRayList;
            }

            List<LightColor> convertedColorList = this.ConvertColor(incoming.lightColor);

            Vector3 perpendicular = inputGate.transform.up;
            float dotPerpendicular = Vector3.Dot(incoming.ray.direction, perpendicular);

            Vector3 reflectedVector = -(outputGate.transform.right * dotNormal) - (outputGate.transform.up * dotPerpendicular);
            LightRay reflectedLightRay = new LightRay();
            reflectedLightRay.ray = new Ray(outputGate.transform.position, reflectedVector);
            reflectedLightRay.lightColor = convertedColorList[0];
            reflectedRayList.Add(reflectedLightRay);
            return reflectedRayList;


        }
    } 
}
