﻿using System.Collections;
using System.Collections.Generic;
using Data;
using UnityEngine;

namespace Interactables
{
    public class Reflector : BaseInteractable
    {
        [SerializeField]
        private GameObject child;

        public override List<LightRay> RetrieveDirectedRayList(LightRay incoming,
            Vector3 rayHitPosition)
        {
            List<LightRay> reflectedRayList = new List<LightRay>();

            Vector3 normal = this.child.transform.right;

            float dotNormal = Vector3.Dot(incoming.ray.direction, normal);
            bool isColorAccepted = this.AcceptColor(incoming.lightColor);

            //Debug.LogWarning(this.name + ": " + isColorAccepted);

            if (dotNormal >= 0 || !isColorAccepted)
            {
                return reflectedRayList;
            }

            List<LightColor> convertedColorList = this.ConvertColor(incoming.lightColor);

            Vector3 perpendicular = this.child.transform.up;

            float dotPerpendicular = Vector3.Dot(incoming.ray.direction, perpendicular);

            Vector3 reflectedVector = (-1*normal*dotNormal) + (perpendicular*dotPerpendicular);

            LightRay reflectedLightRay = new LightRay();
            reflectedLightRay.ray = new Ray(rayHitPosition, reflectedVector);
            reflectedLightRay.lightColor = convertedColorList[0];
            reflectedRayList.Add(reflectedLightRay);

            return reflectedRayList;
        }

        public override bool Raycast(Ray incomingRay, out RaycastHit hitInfo)
        {
            Collider [] childColliders = this.GetComponentsInChildren<Collider>();
            hitInfo = new RaycastHit();
            foreach (Collider childCollider in childColliders)
            {
                if (childCollider.Raycast(incomingRay, out hitInfo, Mathf.Infinity))
                {
                    return true;
                }
            }

            return false;
        }
    }
}