﻿using System.Collections;
using System.Collections.Generic;
using Data;
using UnityEngine;

namespace Interactables
{
    public class Spitter : BaseInteractable
    {
        [SerializeField]
        private GameObject child;

        [SerializeField]
        private float angle = 30;

        public override List<LightRay> RetrieveDirectedRayList(LightRay incoming,
            Vector3 rayHitPosition)
        {
            List<LightRay> reflectedRayList = new List<LightRay>();

            bool isColorAccepted = this.AcceptColor(incoming.lightColor);

            if (!isColorAccepted)
            {
                return reflectedRayList;
            }

            List<LightColor> convertedColorList = this.ConvertColor(incoming.lightColor);

            Vector3 originalRayDirection = incoming.ray.direction;
            Vector3 leftRotated = Quaternion.AngleAxis(-this.angle, Vector3.forward) * originalRayDirection;
            Vector3 rightRotated = Quaternion.AngleAxis(this.angle, Vector3.forward)*originalRayDirection;

            LightRay reflectedLeftLightRay = new LightRay();
            reflectedLeftLightRay.ray = new Ray(rayHitPosition, leftRotated);
            reflectedLeftLightRay.lightColor = convertedColorList[0];
            reflectedRayList.Add(reflectedLeftLightRay);

            LightRay reflectedRightLightRay = new LightRay();
            reflectedRightLightRay.ray = new Ray(rayHitPosition, rightRotated);
            reflectedRightLightRay.lightColor = convertedColorList[1];
            reflectedRayList.Add(reflectedRightLightRay);

            return reflectedRayList;
        }

        public override bool Raycast(Ray incomingRay, out RaycastHit hitInfo)
        {
            Collider[] childColliders = this.GetComponentsInChildren<Collider>();
            hitInfo = new RaycastHit();
            foreach (Collider childCollider in childColliders)
            {
                if (childCollider.Raycast(incomingRay, out hitInfo, Mathf.Infinity))
                {
                    return true;
                }
            }

            return false;
        }
    }
}
