﻿using System;
using System.Collections;
using System.Collections.Generic;
using Data;
using UnityEngine;
using Utilities;

public delegate void PuzzleSolved(Goal goal);

public class Goal : MonoBehaviour
{
    public event PuzzleSolved OnPuzzleSolved;

    [SerializeField]
    private GameObject child;

    [SerializeField]
    private List<LightColor> colorForSuccessList;

    private Dictionary<LightColor, MeshRenderer> colorToRendererMap;

    [SerializeField]
    private Material starMaterial;

    [SerializeField]
    private Material splitterMaterial;

    private List<LightColor> currentHittingColorList;

    private void Awake()
    {
        this.currentHittingColorList = new List<LightColor>();

        this.colorToRendererMap = new Dictionary<LightColor, MeshRenderer>();
        for (int i = 0 ; i < colorForSuccessList.Count ; i++)
        {
            float angle = 360.0f * ((float) i/(float) colorForSuccessList.Count);

            Vector3 starPosition = Quaternion.AngleAxis(angle, Vector3.forward) * Vector3.up * 1.3f;

            LightColor lightColor = colorForSuccessList[i];
            GameObject starObject = GameObject.CreatePrimitive(PrimitiveType.Plane);
            starObject.transform.eulerAngles = new Vector3(90, 0, 0);
            starObject.transform.localScale = Vector3.one * 0.07f;
            starObject.transform.parent = this.child.transform;
            starObject.transform.localPosition = starPosition;
            starObject.name = lightColor.ToString();
            MeshRenderer starMeshRenderer = starObject.GetComponent<MeshRenderer>();
            starMeshRenderer.material = Instantiate<Material>(this.starMaterial);
            starMeshRenderer.material.color = GameUtilities.LightColorDatabase[lightColor];

            GameObject circleObject = GameObject.CreatePrimitive(PrimitiveType.Plane);
            circleObject.transform.eulerAngles = new Vector3(90, 0, 0);
            circleObject.transform.parent = starObject.transform;
            circleObject.transform.localPosition = Vector3.back * 0.1f;
            circleObject.transform.localScale = Vector3.one * 1.6f;
            circleObject.name = lightColor.ToString() + "_bg";
            MeshRenderer starBgMeshRenderer = circleObject.GetComponent<MeshRenderer>();
            starBgMeshRenderer.material = Instantiate<Material>(this.splitterMaterial);
            Color bgColor = Color.white;
            bgColor.a = 0.4f;
            starBgMeshRenderer.material.color = bgColor;

            starBgMeshRenderer.enabled = false;
            this.colorToRendererMap.Add(lightColor, starBgMeshRenderer);
        }
    }

    public void TestSuccessCase(List<LightRay> lightRayList)
    {
        this.currentHittingColorList.Clear();
        foreach (LightRay lightRay in lightRayList)
        {
            RaycastHit hit;
            if (this.Raycast(lightRay.ray, out hit))
            {
                this.currentHittingColorList.Add(lightRay.lightColor);
            }
        }

        //string str = String.Empty;
        //foreach (LightColor lightColor in currentHittingColorList)
        //{
        //    str += lightColor + " ";
        //}
        //Debug.LogWarning("+++" + str + "+++");

        foreach (MeshRenderer renderer in colorToRendererMap.Values)
        {
            renderer.enabled = false;
        }

        List<LightColor> tmpColorForSuccessList = new List<LightColor>(this.colorForSuccessList);
        foreach (LightColor lightColor in currentHittingColorList)
        {
            tmpColorForSuccessList.Remove(lightColor);
            this.colorToRendererMap[lightColor].enabled = true;
            //str += lightColor + " ";
        }

        float glowForStarColor = ((float) (colorForSuccessList.Count - tmpColorForSuccessList.Count)/(float) (colorForSuccessList.Count + 1));
        Color starColor = Color.white * glowForStarColor;
        starColor.a = 1.0f;
        starMaterial.color = starColor;

        //str = String.Empty;
        //foreach (LightColor lightColor in tmpColorForSuccessList)
        //{
        //    str += lightColor + " ";
        //}
        //Debug.LogWarning("---" + str + "---");

        if (tmpColorForSuccessList.Count == 0)
        {
            if (this.OnPuzzleSolved != null)
            {
                this.OnPuzzleSolved(this);
            }
        }
    }

    private bool Raycast(Ray incomingRay, out RaycastHit hitInfo)
    {
        Collider[] childColliders = this.GetComponentsInChildren<Collider>();
        hitInfo = new RaycastHit();
        foreach (Collider childCollider in childColliders)
        {
            if (childCollider.Raycast(incomingRay, out hitInfo, Mathf.Infinity))
            {
                return true;
            }
        }

        return false;
    }
}
